// k_tssi.hxx
//
// Mical, a regular grammar inference library
// Copyright (C) 2003 Maxime Rey
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef MICAL_K_TSSI_HXX_
#define MICAL_K_TSSI_HXX_

#include <mical/algorithms/k_tssi.hh>
#include <mical/internal/k_window.hh>

#include <mical/tools/predefs.hh>

#include <map>

namespace mical {

  using namespace vcsn;
  using std::set;
  using std::map;

  template <typename A, typename T, typename Container_>
  void
  k_tssi(Element<A, T>& a, 
	 unsigned k, 
	 const FixedSample<Container_,
	 typename Element<A, T>::monoid_elt_t>& sample)
  {
    ///////////////////////////////////////////////////    
    typedef typename Element<A, T>::monoid_elt_t word_t;

    typedef 
      FixedSample<Container_,
      typename Element<A, T>::monoid_elt_t> sample_t;
    ///////////////////////////////////////////////////    

    KWindow<typename word_t::set_t::letter_t> win(k);
    
    map<word_t, hstate_t> mappy;
    
    hstate_t first = a.add_state();
    a.set_initial(first);
    
    mappy[word_t()] = first;
    unsigned count = 0;
    for (typename sample_t::sample_iterator i = sample.begin(); 
	 i != sample.end(); i++)
      {
	win.clear();
	add_word(a, i->begin(), first, win, mappy);
	// SAVE_AUTOMATON_DOT_SIMPLE(count, a);
	count++;
      }
  }
  
  template <typename A, typename T, typename Word>
  void
  add_word(Element<A, T>& a, 
	   typename Word::const_iterator p, 
	   hstate_t current,
	   KWindow<typename Word::set_t::letter_t>& k_window,
	   map<Word, hstate_t>& tab)
  {
    typename map<Word, hstate_t>::iterator res;

    if (*p != 0)
      {
	k_window.insert(*p);

	if ((res = tab.find(k_window.get_window(Word()))) == tab.end())
	  {
	    hstate_t newo = a.add_state();
	
	    tab[k_window.get_window(Word())] = newo;
	
	    add_letter_edge(a, current, newo, *p);
	
	    add_word(a, ++p, newo, k_window, tab);
	  }
	else
	  {
	    add_letter_edge(a, current, res->second, *p);
	    add_word(a, ++p, res->second, k_window, tab);
	  }
      }
    else
      a.set_final(current);
  }


} // mical

#endif // MICAL_K_TSSI_HXX_ 



