// mggi.hxx
//
// Mical, a regular grammar inference library
// Copyright (C) 2003 Maxime Rey
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef MICAL_MGGI_HXX_
#define MICAL_MGGI_HXX_

#include <mical/algorithms/mggi.hh>

#include <vaucanson/algebra/concrete/free_monoid/predefs.hh>

#include <utility>

namespace mical {

  using std::map;
  using namespace vcsn;
  using namespace vcsn::algebra;
  using namespace vcsn::algebra::char_letter;

  template<typename A, typename T, typename Alphabet_>
  Element<A, T>
  change_alphabet(const Element<A, T>& a, 
		  map<typename Alphabet::letter_t, 
		  typename Alphabet::letter_t> mappy,
		  Alphabet_ alpha)
  {
    Element<A, T> res = a;
    res.emancipate();
    
    res.series().monoid().alphabet() = alpha;

    for (typename Element<A, T>::edge_iterator i = res.edges().begin();
	 i != res.edges().end(); i++)
      res.del_edge(*i);

    for (typename Element<A, T>::edge_iterator i = a.edges().begin();
	 i != a.edges().end(); i++)
      res.add_letter_edge(a.origin_of(*i), a.aim_of(*i), 
			  mappy[*a.label_of(*i).begin()->first.begin()]);

    return res;
  }


} // mical

#endif // MICAL_MGGI_HXX_
