// fission_sup_semi_lattice.hxx
//
// Mical, a regular grammar inference library
// Copyright (C) 2003 Maxime Rey
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef MICAL_FISSION_SUP_SEMI_LATTICE_HXX_
#define MICAL_FISSION_SUP_SEMI_LATTICE_HXX_

#include <mical/framework/fission_sup_semi_lattice.hh>

namespace mical {
  
  template <class A, class T, class Partition_>
  void
  proxy_fission(search_kind::sup_semi_lattice, 
		const Element<A, T>& view, 
		Partition<Partition_>& partition, 
		hstate_t lhs, 
		hstate_t rhs)
  {
    typedef Element<A, T>            automaton_t;
    
    partition.extract(lhs, rhs);
    
    bool flag = false;
    for (typename automaton_t::alphabet_t::iterator 
	   i = view.alphabet().begin();
	 i != view.alphabet().end(); i++)
      {
	set<hedge_t> delta_edge1, delta_edge2;
	view.letter_deltac(delta_edge1, lhs, *i, delta_kind::edges());
	view.letter_deltac(delta_edge1, rhs, *i, delta_kind::edges());
	
	if ((delta_edge1.size() > 1) || (delta_edge2.size() > 1))
	  {
	    flag = true;
	    break;
	  }
      }
    
    if (flag)
      partition.weld(lhs, rhs);
  }
  
} // mical

#endif // MICAL_FISSION_SUP_SEMI_LATTICE_HXX_
