// generate.hxx
//
// Mical, a regular grammar inference library
// Copyright (C) 2003 Maxime Rey
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef MICAL_GENERATE_HXX_
# define MICAL_GENERATE_HXX_

#include <mical/framework/generate.hh>

#include <utility>

namespace mical {

  using std::map;

  template <class A, class T, class B, class U>
  void
  generate(const Element<A, T>& lhs, Element<B, U>& rhs)
  {
    map<hstate_t, hstate_t> assoc;
   
    rhs.series().monoid().alphabet() = lhs.alphabet();
    
    for (typename Element<A, T>::state_iterator i = lhs.states().begin(); 
	 i != lhs.states().end(); i++)
      assoc[*i] = rhs.add_state();

    for (typename Element<A, T>::state_iterator i = lhs.states().begin(); 
	 i != lhs.states().end(); i++)
      {
        if (lhs.is_final(*i))
          rhs.set_final(assoc[*i]);
        if (lhs.is_initial(*i))
          rhs.set_initial(assoc[*i]);
      }
    
    for (typename Element<A, T>::edge_iterator i = lhs.edges().begin(); 
	 i != lhs.edges().end(); i++)
      rhs.add_edge(assoc[lhs.origin_of(*i)], assoc[lhs.aim_of(*i)], lhs.label_of(*i));
  }
  
} // mical

#endif // MICAL_GENERATE_HXX_
