// partition_set.hxx
//
// Mical, a regular grammar inference library
// Copyright (C) 2003 Maxime Rey
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef MICAL_PARTITION_SET_HXX_
# define MICAL_PARTITION_SET_HXX_

#include <set>

#include <mical/framework/partitions/concrete/partition_set.hh>

namespace mical {
  
  block_t 
  single(elt_t s)
  {
    block_t newo;
    
    newo.insert(s);
    return newo;
  }

  void
  proxy_implose(partition_t& partition, 
		const block_iterator& i)
  {
    for (elt_iterator j = i->begin();
	 j != i->end(); j++)
      partition.insert(single(*j));
    partition.erase(*i);
  }


  void 
  proxy_extract(partition_t& partition,
		const block_iterator& block, 
		elt_t elt)
  {
    block_t tmp = *block;
    tmp.erase(elt);

    partition.insert(single(elt));
    partition.insert(tmp);
    partition.erase(*block);
  }
  

  void 
  proxy_extract(partition_t& partition,
		const block_iterator& block, 
		const block_t& target)
  {
    block_t tmp = *block;
    for (elt_iterator i = target.begin(); i != target.end(); i++)
      tmp.erase(*i);
    partition.insert(tmp);
    partition.erase(*block);

    partition.insert(target);
  }


  void 
  proxy_weld(partition_t& partition,
	     const block_iterator& block, 
	     const block_iterator& target)
  {
    block_t tmp = *block;
    for (elt_iterator i = target->begin(); i != target->end(); i++)
      tmp.insert(*i);
    partition.insert(tmp);
    partition.erase(*block);

    partition.erase(*target);
  }


  void 
  proxy_insert(partition_t& partition,
	       const block_t& block)
  {
    partition.insert(block);
  }

  void 
  proxy_insert(partition_t& partition,
	       elt_t elt)
  {
    partition.insert(single(elt));
  }

  void 
  proxy_insert(partition_t& partition,
	       const block_iterator& block, 
	       elt_t elt)
  {
    block_t tmp = *block;
    tmp.insert(elt);
    
    partition.insert(tmp);
    partition.erase(*block);
  }

  void 
  proxy_erase(partition_t& partition,
	      const block_iterator& block)
  {
    partition.erase(*block);
  }

  void 
  proxy_erase(partition_t& partition,
	      const block_iterator& block, 
	      elt_t elt)
  {
    block_t tmp = *block;
    tmp.erase(elt);
    
    partition.insert(tmp);
    partition.erase(*block);
  }

} // mical

#endif // MICAL_PARTITION_SET_HXX_
