// partitions.hh
//
// Mical, a regular grammar inference library
// Copyright (C) 2003 Maxime Rey
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef MICAL_PARTITIONS_HH_
#define MICAL_PARTITIONS_HH_

#include <set>
#include <vaucanson/automata/concept/automata.hh>

namespace mical {

  using namespace vcsn;
  using std::set;

  template <class A>
  struct partition_traits
  {
    typedef A      type_t;
  };

  template <class A>
  struct partition_traits<set<A> >
  {
    typedef set<A> self_t;

    typedef A      sub_type_t;

    typedef typename partition_traits<A>::type_t type_t;
  };

  template <class A>
  struct Partition
  {};

  template <class A>
  struct Partition<partition_traits<A> >
  {
    typedef typename partition_traits<A>::self_t        partition_t;
    typedef typename partition_traits<A>::sub_type_t    block_t;
    typedef typename partition_traits<A>::type_t        elt_t;
    
    typedef typename partition_t::iterator              block_iterator;
    typedef typename partition_t::const_iterator        block_const_iterator;
    
    Partition();

    size_t                 size() const;
    bool                   empty() const;
    
    
    block_iterator               begin();
    block_const_iterator         begin() const;
    
    block_iterator               end();
    block_const_iterator         end() const;

    void clear();
    
    block_iterator find(const block_t& block) const;
    block_iterator find(elt_t elt) const;
    
    block_iterator find_elt(elt_t elt) const;

    void implose(const block_iterator& block);
    
    void extract(const block_iterator& block, elt_t elt);
    void extract(const block_iterator& block, const block_t& target);
    void extract(const block_t& block, elt_t elt);
    void extract(const block_t& block, const block_t& target);
    void extract(elt_t block, elt_t target);
    void extract(elt_t block, const block_t& target);
    
    void weld(const block_iterator& lhs, const block_iterator& rhs);
    void weld(const block_t& lhs, const block_t& rhs);
    void weld(const block_t& lhs, elt_t rhs);
    void weld(elt_t lhs, elt_t rhs);

    void insert(const block_t& block);
    void insert(elt_t elt);
    void insert(const block_iterator& block, elt_t elt);
    
    void erase(const block_iterator& block);
    void erase(const block_iterator& block, elt_t elt);

    partition_t& partition();

    Partition<partition_traits<A> >&
    operator=(Partition<partition_traits<A> > other);

  private:
    
    partition_t partition_;

  };

  
  template <template <class> class Partition_, 
	    template <class> class Block_, 
	    typename Elt_>
  bool 
  operator==(Partition
	     <partition_traits<Partition_<Block_<Elt_> > > > lhs,
	     Partition 
	     <partition_traits<Partition_<Block_<Elt_> > > > rhs);


  template <template <class> class Partition_, 
	    template <class> class Block_, 
	    typename Elt_>
  bool 
  operator!=(Partition
	     <partition_traits<Partition_<Block_<Elt_> > > > lhs,
	     Partition 
	     <partition_traits<Partition_<Block_<Elt_> > > > rhs);

} // mical

#include <mical/framework/partitions/partitions.hxx>

#endif // MICAL_PARTITIONS_HH_
