// partitions.hxx
//
// Mical, a regular grammar inference library
// Copyright (C) 2003 Maxime Rey
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef MICAL_PARTITIONS_HXX_
#define MICAL_PARTITIONS_HXX_

#include <mical/framework/partitions/partitions.hh>
#include <mical/tools/macros.hh>

#include <mical/framework/partitions/concrete/partition_set.hh> 

namespace mical {

  template <class A>
  Partition<partition_traits<A> >::
  Partition()
  {}

  template <class A>
  size_t                
  Partition<partition_traits<A> >::
  size() const
  {
    return partition_.size();
  }

  template <class A>
  bool
  Partition<partition_traits<A> >::
  empty() const
  {
    return partition_.empty();
  }
    
    
  template <class A>
  typename partition_traits<A>::self_t::iterator               
  Partition<partition_traits<A> >::
  begin()
  {
    return partition_.begin();
  }

  template <class A>
  typename partition_traits<A>::self_t::const_iterator         
  Partition<partition_traits<A> >::
  begin() const
  {
    return partition_.begin();
  }
        
  template <class A>
  typename partition_traits<A>::self_t::iterator              
  Partition<partition_traits<A> >::
  end()
  {
    return partition_.end();
  }

  template <class A>
  typename partition_traits<A>::self_t::const_iterator         
  Partition<partition_traits<A> >::
  end() const
  {
    return partition_.end();
  }

  template <class A>
  void 
  Partition<partition_traits<A> >::
  clear()
  {
    partition_.clear();
  }

  template <class A>
  typename partition_traits<A>::self_t::iterator                
  Partition<partition_traits<A> >::
  find(const block_t& block) const
  {
    return std::find(partition_.begin(), partition_.end(), block);
  }
   
  template <class A>
  typename partition_traits<A>::self_t::iterator              
  Partition<partition_traits<A> >::
  find(elt_t elt) const
  {
    return std::find(partition_.begin(), partition_.end(), single(elt));
  }
    
  template <class A>
  typename partition_traits<A>::self_t::iterator              
  Partition<partition_traits<A> >::
  find_elt(elt_t elt) const
  {
    for (block_iterator i = partition_.begin();
	 i != partition_.end(); i++)
      {
	bool flag = false;
	for (typename block_t::iterator j = i->begin(); j != i->end(); j++)
	  {
	    if (*j == elt)
	      flag = true; 
	  }
	if (flag)
	  return i;
      }
    return partition_.end();
  }


  template <class A>
  void 
  Partition<partition_traits<A> >::
  implose(const block_iterator& block)
  {
    if (block != partition_.end())
      PROXY2(implose, partition_, block);
  }

  template <class A>
  void 
  Partition<partition_traits<A> >::
  extract(const block_iterator& block, elt_t elt)
  {
    if (block != partition_.end())
      PROXY3(extract, partition_, block, elt);
  }
  
  template <class A>
  void 
  Partition<partition_traits<A> >::
  extract(const block_iterator& block, const block_t& target)
  {
    if (block != partition_.end())
      PROXY3(extract, partition_, block, target);
  }

  template <class A>
  void 
  Partition<partition_traits<A> >::
  extract(const block_t& block, elt_t elt)
  {
    extract(find(block), elt);
  }
  
  template <class A>
  void 
  Partition<partition_traits<A> >::
  extract(const block_t& block, const block_t& target)
  {
    extract(find(block), target);
  }

  template <class A>
  void 
  Partition<partition_traits<A> >::
  extract(elt_t block, elt_t target)
  {
    extract(find_elt(block), target);
  }
  
  template <class A>
  void 
  Partition<partition_traits<A> >::
  extract(elt_t block, const block_t& target)
  {
    extract(find_elt(block), target);
  }


  template <class A>
  void 
  Partition<partition_traits<A> >::
  weld(const block_iterator& lhs, const block_iterator& rhs)
  {
    if ((lhs != partition_.end()) && (rhs != partition_.end()))
      PROXY3(weld, partition_, lhs, rhs);
  }

  template <class A>
  void 
  Partition<partition_traits<A> >::
  weld(const block_t& lhs, const block_t& rhs)
  {
    weld(find(lhs), find(rhs));
  }

  template <class A>
  void 
  Partition<partition_traits<A> >::
  weld(const block_t& lhs, elt_t rhs)
  {
    weld(find(lhs), find_elt(rhs));
  }

  template <class A>
  void 
  Partition<partition_traits<A> >::
  weld(elt_t lhs, elt_t rhs)
  {
    weld(find_elt(lhs), find_elt(rhs));
  }

  template <class A>
  void 
  Partition<partition_traits<A> >::
  insert(const block_t& block)
  {
    PROXY2(insert, partition_, block);
  }

  template <class A>
  void 
  Partition<partition_traits<A> >::
  insert(elt_t elt)
  {
    PROXY2(insert, partition_, elt);
  }

  template <class A>
  void 
  Partition<partition_traits<A> >::
  insert(const block_iterator& block, elt_t elt)
  {
    PROXY3(insert, partition_, block, elt);
  }

  template <class A>
  void 
  Partition<partition_traits<A> >::
  erase(const block_iterator& block)
  {
    PROXY2(erase, partition_, block);
  }

  template <class A>
  void 
  Partition<partition_traits<A> >::
  erase(const block_iterator& block, elt_t elt)
  {
    PROXY3(erase, partition_, block, elt);
  }

  template <class A>
  typename partition_traits<A>::self_t&
  Partition<partition_traits<A> >::
  partition()
  {
    return partition_;
  }

  template <class A>
  Partition<partition_traits<A> >&
  Partition<partition_traits<A> >::
  operator=(Partition<partition_traits<A> > other)
  {
    this->partition_ = other.partition();

    return *this;
  }


  template <template <class> class Partition_, 
	    template <class> class Block_, 
	    typename Elt_>
  bool 
  operator==(Partition
	     <partition_traits<Partition_<Block_<Elt_> > > > lhs,
	     Partition 
	     <partition_traits<Partition_<Block_<Elt_> > > > rhs)
  {
//     typedef typename Partition<partition_traits<Partition_<Block_<Elt_> > > >::
//       block_iterator iterator;
    
//     iterator j = rhs.begin();

//     if (lhs.size() != rhs.size())
//       return false;
      
//     for (iterator i = lhs.begin(); i != lhs.end(); i++, j++)
//       if (*i != *j)
// 	return false;

//     return true;
    Partition<partition_traits<Partition_<Block_<Elt_> > > >
      tmp1 = lhs, 
      tmp2 = rhs;

    return (tmp1.partition() == tmp2.partition());
  }

  
  template <template <class> class Partition_, 
	    template <class> class Block_, 
	    typename Elt_>
  bool 
  operator!=(Partition
	     <partition_traits<Partition_<Block_<Elt_> > > > lhs,
	     Partition 
	     <partition_traits<Partition_<Block_<Elt_> > > > rhs)
  {
    return !(lhs == rhs);
  }

} // mical



#endif // MICAL_PARTITIONS_HXX_
