// make_mca.hxx
//
// Mical, a regular grammar inference library
// Copyright (C) 2003 Maxime Rey
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef MICAL_MAKE_MCA_HXX_
#define MICAL_MAKE_MCA_HXX_

#include <mical/internal/make_mca.hh>

namespace mical {

  using namespace vcsn;
  using namespace vcsn::algebra;

  template<typename A, typename T, typename Container_>
  void
  make_mca(Element<A, T>& a, 
	   const FixedSample<Container_,
	   typename Element<A, T>::monoid_elt_t>& sample)
  {
    typedef typename Element<A, T>::monoid_elt_t  word_t;
    typedef  
      FixedSample<Container_, typename Element<A, T>::monoid_elt_t>
      sample_t;

    a.set_initial(a.add_state());

    for (typename sample_t::sample_const_iterator i = sample.begin(); 
	 i != sample.end(); ++i)
      {
	hstate_t from = *a.initial().begin();    
	hstate_t to;
	for (typename word_t::const_iterator j = i->begin(); j != i->end(); ++j)
	  {
	    to = a.add_state();
	    a.add_letter_edge(from, to, *j);
	    from = to;
	  }
	a.set_final(to);
      }
      
  }
  
  
  template<typename A, typename T, typename Kind_, typename Container_>
  void
  make_mca(Element<A, T>& a, 
	   const Presentation<Kind_, Container_,
	   typename Element<A, T>::monoid_elt_t>& sample)
  {
    typedef typename Element<A, T>::monoid_elt_t  word_t;
    typedef  
      Presentation<Kind_, Container_, typename Element<A, T>::monoid_elt_t>
      sample_t;

    a.set_initial(a.add_state());

    for (typename sample_t::sample_const_iterator i = sample.p_begin(); 
	 i != sample.p_end(); ++i)
      {
	hstate_t from = *a.initial().begin();    
	hstate_t to;
	for (typename word_t::const_iterator j = i->begin(); j != i->end(); ++j)
	  {
	    to = a.add_state();
	    a.add_letter_edge(from, to, *j);
	    from = to;
	  }
	a.set_final(to);
      }
      
  }

} // mical

#endif // MICAL_MAKE_MCA_HXX_
