// compute_all.hxx
//
// Mical, a regular grammar inference library
// Copyright (C) 2003 Maxime Rey
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef MICAL_COMPUTE_ALL_HXX_
#define MICAL_COMPUTE_ALL_HXX_

#include <mical/primitives/compute_all.hh>

namespace mical {

  template <class A, class T>
  bool    
  compute_all(Element<A, T> a, 
	      typename Element<A, T>::monoid_elt_t word)
  {
    Compute_n<A, T, typename Element<A, T>::monoid_elt_t> compute;

    return compute(a, word, 1, compute_kind::boolean());
  }

  template <class A, class T, class Container_>
  void    
  compute_all(Element<A, T> a, 
	      typename Element<A, T>::monoid_elt_t word, 
	      Container_& res)
  {
    Compute_n<A, T, typename Element<A, T>::monoid_elt_t> compute;
    bool flag;
    unsigned count = 1;
    unsigned prev_value;

    do 
      {
	prev_value = res.size();
	compute(a, word, count, res);
	flag = (res.size() != prev_value);
	count++;
      }
    while (flag);
  }

} // mical

#endif // MICAL_COMPUTE_ALL_HXX_
