// find_suffix.hx
//
// Mical, a regular grammar inference library
// Copyright (C) 2003 Maxime Rey
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef MICAL_FIND_SUFFIX_HXX_
#define MICAL_FIND_SUFFIX_HXX_

// #include <vaucanson/algorithms/trim.hh>

#include <mical/primitives/find_suffix.hh>
#include <mical/walkers/info.hh>
#include <mical/tools/choose.hh>

#include <vector>

namespace mical {

 using std::vector;

  template<class A, class T, class Word_> 
  struct FindSuffix
  {
    typedef FindSuffix<A, T, Word_>         self_t;
    
    typedef InfoDepth<A, T, self_t>       info_t;

    typedef Word_                         word_t;
    
    typedef vector<hstate_t>              delta_state_t;

    FindSuffix()
    {}

    word_t operator ()(const Element<A, T>& a, 
		       hstate_t state)
    {
      str = "";

      Element<A, T> work = a;
      work.create();

      //      trim(work);

      if (work.states().size())
	depth_search(work, state, *this, security::safe_t(1));
      
      return str;
    }

    void prefix(info_t& info)
    {
      if (!info.first_)
	str = str * info.auto_.label_of(info.edge_).begin()->first;
      
      if (info.auto_.is_final(info.state_))
	info.stop_ = true;
    }

    void infix(info_t& info)
    {}

    void postfix(info_t& info)
    {
      if (info.stop_)
	return;

      if (str.value().size() > 0)
	str.value().erase(--str.end());
    }

    template<typename Container_>
    void 
    delta(info_t& info, Container_& res, hstate_t from)
    {
      info.auto_.deltac(res, from, delta_kind::states());
    }

  private:

    word_t str;
  };


  template <class A, class T>
  typename Element<A, T>::monoid_elt_t
  find_suffix(const Element<A, T>& a, hstate_t state)
  {
    FindSuffix<A, T, typename Element<A, T>::monoid_elt_t> fs;

    return fs(a, state);
  }

} // mical

#endif // MICAL_FIND_SUFFIX_HXX_
