// quotient.hxx
//
// Mical, a regular grammar inference library
// Copyright (C) 2003 Maxime Rey
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef MICAL_QUOTIENT_HXX_
# define MICAL_QUOTIENT_HXX_

#include <mical/primitives/quotient.hh>

#include <mical/walkers/depth_search.hh>

#include <vector>

namespace mical {

  using std::vector;

  template<class A, class T, class Word_> 
  struct Quotient
  {
    typedef Quotient<A, T, Word_>         self_t;
    
    typedef InfoDepth<A, T, self_t>       info_t;

    typedef Word_                         word_t;
    
    typedef vector<hstate_t>              delta_state_t;

    Quotient()
    {}

    hstate_t operator ()(const Element<A, T>& a, 
			 word_t word)
    {
      str = word;

      if (str.value().size() == 0)
	return *a.initial().begin();

      p = str.begin();

      depth_search(a, *this, security::responsable_t());

      return state_;
    }

    void prefix(info_t& info)
    {
      if ((!info.first_) &&
	  (find(str.begin(), str.end(), *p) == str.end()))
	{
	  state_ = info.state_;
	  info.stop_ = true;
	}
    }

    void infix(info_t& info)
    {}

    void postfix(info_t& info)
    {}

    template<typename Container_>
    void 
    delta(info_t& info, Container_& res, hstate_t from)
    {
      info.auto_.letter_deltac(res, from, *p, delta_kind::states());
      p++;
    }

  private:

    typename word_t::iterator p;
    word_t str;

    hstate_t state_;
  };

  template <class A, class T>
  hstate_t
  quotient(const Element<A, T>& a, 
	   typename Element<A, T>::monoid_elt_t word)
  {
    Quotient<A, T, typename Element<A, T>::monoid_elt_t> qt;

    return qt(a, word);
  }

} // mical

#endif // MICAL_QUOTIENT_HXX_
