// set_sample.hxx
//
// Mical, a regular grammar inference library
// Copyright (C) 2003 Maxime Rey
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef MICAL_SET_SAMPLE_HXX_
# define MICAL_SET_SAMPLE_HXX_

#include <set>
#include <algorithm>

#include <mical/samples/kind/concrete/set_sample.hh>

namespace mical {

  using std::set;
  using std::find;

  template <typename Word_>
  void 
  proxy_add_word(const Word_& word, set<Word_>& container)
  {
    container.insert(word);
  }

  template <typename Word_>
  void 
  proxy_del_word(const Word_& word, set<Word_>& container)
  {
    typedef typename set<Word_>::iterator iterator_t;

    iterator_t d;
    if ((d = find(container.begin(), container.end(), word))
	!= container.end())
      container.erase(d);
  }

  template <typename Word_>
  size_t 
  proxy_size(const set<Word_>& container)
  {
    return container.size();
  }

  template <typename Word_>
  bool 
  proxy_empty(const set<Word_>& container)
  {
    return container.empty();
  }

  template <typename Word_>
  typename set<Word_>::iterator
  proxy_begin(const set<Word_>& container, kind_iterator::not_const)
  {
    return (typename set<Word_>::iterator) container.begin();
  }

  template <typename Word_>
  typename set<Word_>::const_iterator
  proxy_begin(const set<Word_>& container, kind_iterator::not_mutable)
  {
    return (typename set<Word_>::const_iterator) container.begin();
  }

  template <typename Word_>
  typename set<Word_>::reverse_iterator
  proxy_rbegin(const set<Word_>& container, kind_iterator::not_const)
  {
     return (typename set<Word_>::reverse_iterator) container.rbegin();
  }

  template <typename Word_>
  typename set<Word_>::const_reverse_iterator
  proxy_rbegin(const set<Word_>& container, kind_iterator::not_mutable)
  {
    return (typename set<Word_>::const_reverse_iterator) container.rbegin();
  }

  template <typename Word_>
  typename set<Word_>::iterator
  proxy_end(const set<Word_>& container, kind_iterator::not_const)
  {
    return (typename set<Word_>::iterator) container.end();
  }

  template <typename Word_>
  typename set<Word_>::const_iterator
  proxy_end(const set<Word_>& container, kind_iterator::not_mutable)
  {
    return (typename set<Word_>::const_iterator) container.end();
  }

  template <typename Word_>
  typename set<Word_>::reverse_iterator
  proxy_rend(const set<Word_>& container, kind_iterator::not_const)
  {
    return (typename set<Word_>::reverse_iterator) container.rend();
  }

  template <typename Word_>
  typename set<Word_>::const_reverse_iterator
  proxy_rend(const set<Word_>& container, kind_iterator::not_mutable)
  {
    return (typename set<Word_>::const_reverse_iterator) container.rend();
  }

  template <typename Word_>
  void 
  proxy_clear(set<Word_>& container)
  {
    container.clear();
  }

} // mical

#endif // MICAL_SET_SAMPLE_HXX_
