// presentation.hh
//
// Mical, a regular grammar inference library
// Copyright (C) 2003 Maxime Rey
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef MICAL_PRESENTATION_HH_
# define MICAL_PRESENTATION_HH_

#include <unistd.h>
#include <mical/tools/macros.hh>
#include <mical/samples/kind/sample.hh>

#include <mical/framework/framework_kind.hh>

namespace mical {


  template <typename Kind_, typename Container_, typename Word_>
  struct Presentation
  {
    ///////////////////////////////////////////////////////////////////////////
    typedef Container_                               container_t;
    typedef Word_                                    word_t;
    typedef Kind_                                    kind_t;
    
    typedef typename container_t::iterator           sample_iterator;
    typedef typename container_t::const_iterator     sample_const_iterator;

    typedef typename container_t::reverse_iterator   sample_reverse_iterator;
    ///////////////////////////////////////////////////////////////////////////

    ///////////////////////////////////////////////////////////////////////////
    Presentation();
    
    void                          add_positive(const word_t& word);
    void                          add_negative(const word_t& word);

    void                          del_positive(const word_t& word);
    void                          del_negative(const word_t& word);
    
    size_t                        size_positive() const;
    size_t                        size_negative() const;
    size_t                        size() const;

    bool                          empty_positive() const;
    bool                          empty_negative() const;
    bool                          empty() const;

    sample_iterator               p_begin();
    sample_const_iterator         p_begin() const;
    sample_reverse_iterator       p_rbegin();
    
    sample_iterator               n_begin();
    sample_const_iterator         n_begin() const;
    sample_reverse_iterator       n_rbegin();

    sample_iterator               p_end();
    sample_const_iterator         p_end() const;
    sample_reverse_iterator       p_rend();

    sample_iterator               n_end();
    sample_const_iterator         n_end() const;
    sample_reverse_iterator       n_rend();

    void                          clear_positive();
    void                          clear_negative();
    void                          clear();

    container_t                   positive() const;
    container_t                   negative() const;
    ///////////////////////////////////////////////////////////////////////////

    void                          init();

    void                          next_step();

    container_t&                  positive_container();
    container_t&                  negative_container();
    
    Presentation<Kind_, Container_, Word_>&
    operator=(const Presentation<Kind_, Container_, Word_>& other);

  private:
    
    container_t positive_;
    container_t negative_;

    container_t positive_step_;
    container_t negative_step_;

    unsigned count_neg;
    unsigned count_pos;
  };


  template <typename Container_, typename Word_>
  struct Presentation<inference_kind::fixed, Container_, Word_>
  {
    ///////////////////////////////////////////////////////////////////////////
    typedef Container_                               container_t;
    typedef Word_                                    word_t;
    typedef inference_kind::fixed                    kind_t;
    
    typedef typename container_t::iterator           sample_iterator;
    typedef typename container_t::const_iterator     sample_const_iterator;

    typedef typename container_t::reverse_iterator   sample_reverse_iterator;
    ///////////////////////////////////////////////////////////////////////////

    ///////////////////////////////////////////////////////////////////////////
    Presentation();
    
    void                          add_positive(const word_t& word);
    void                          add_negative(const word_t& word);

    void                          del_positive(const word_t& word);
    void                          del_negative(const word_t& word);
    
    size_t                        size_positive() const;
    size_t                        size_negative() const;
    size_t                        size() const;


    bool                          empty_positive() const;
    bool                          empty_negative() const;
    bool                          empty() const;


    sample_iterator               p_begin();
    sample_const_iterator         p_begin() const;
    sample_reverse_iterator       p_rbegin();
    
    sample_iterator               n_begin();
    sample_const_iterator         n_begin() const;
    sample_reverse_iterator       n_rbegin();

    sample_iterator               p_end();
    sample_const_iterator         p_end() const;
    sample_reverse_iterator       p_rend();

    sample_iterator               n_end();
    sample_const_iterator         n_end() const;
    sample_reverse_iterator       n_rend();

    void                          clear_positive();
    void                          clear_negative();
    void                          clear();

    container_t                   positive() const;
    container_t                   negative() const;
    ///////////////////////////////////////////////////////////////////////////

    container_t&                  positive_container();
    container_t&                  negative_container();

    Presentation<inference_kind::fixed, Container_, Word_>&
    operator=(const Presentation<inference_kind::fixed, Container_, Word_>& other);

  private:
    
    container_t positive_;
    container_t negative_;
  };


  // Operators' overloading

  template <typename Kind_, typename Container_, typename Word_>
  bool 
  operator==(const Presentation<Kind_, Container_, Word_>& lhs,
	     const Presentation<Kind_, Container_, Word_>& rhs);

  template <typename Kind_, typename Container_, typename Word_>
  bool 
  operator!=(const Presentation<Kind_, Container_, Word_>& lhs,
	     const Presentation<Kind_, Container_, Word_>& rhs);
  
} // mical

#include <mical/samples/kind/presentation.hxx>

#endif // MICAL_PRESENTATION_HH_
