// fixed_sample.hh
//
// Mical, a regular grammar inference library
// Copyright (C) 2003 Maxime Rey
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef MICAL_FIXED_SAMPLE_HH_
# define MICAL_FIXED_SAMPLE_HH_

#include <unistd.h>
#include <mical/tools/macros.hh>

namespace mical {

  namespace kind_iterator
  {
    struct not_mutable
    {};
    
    struct not_const
    {};

  } // kind_iterator

  template <typename Container_, typename Word_>
  struct FixedSample
  {
    ///////////////////////////////////////////////////////////////////////////
    typedef Container_                               container_t;
    typedef Word_                                    word_t;
    
    typedef typename container_t::iterator           sample_iterator;
    typedef typename container_t::const_iterator     sample_const_iterator;
        
    typedef typename container_t::reverse_iterator   sample_reverse_iterator;
    ///////////////////////////////////////////////////////////////////////////

    ///////////////////////////////////////////////////////////////////////////
    FixedSample();
    
    void                          add_word(const word_t& word);
    void                          del_word(const word_t& word);

    size_t                        size() const;
    bool                          empty() const;

    sample_iterator               begin();
    sample_const_iterator         begin() const;
    sample_reverse_iterator       rbegin();
    
    sample_iterator               end();
    sample_const_iterator         end() const;
    sample_reverse_iterator       rend();

    void                          clear();

    container_t                  words() const;
    ///////////////////////////////////////////////////////////////////////////

    container_t&                 container();

    FixedSample<Container_, Word_>&
    operator=(const FixedSample<Container_, Word_>& other);

  private:
    
    container_t container_;
  };

  // Operators' overloading

  template <typename Container_, typename Word_>
  bool 
  operator==(const FixedSample<Container_, Word_>& lhs,
	     const FixedSample<Container_, Word_>& rhs);
	     
  // in the meaning of inclusion
  template <typename Container_, typename Word_>
  bool
  operator<=(const FixedSample<Container_, Word_>& lhs,
	     const FixedSample<Container_, Word_>& rhs);

  // in the meaning of inclusion
  template <typename Container_, typename Word_>
  bool
  operator>=(const FixedSample<Container_, Word_>& lhs,
	     const FixedSample<Container_, Word_>& rhs);

} // mical

#include <mical/samples/kind/sample.hxx>

#endif // MICAL_FIXED_SAMPLE_HH_
