// fixed_sample.hxx
//
// Mical, a regular grammar inference library
// Copyright (C) 2003 Maxime Rey
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef MICAL_FIXED_SAMPLE_HXX_
# define MICAL_FIXED_SAMPLE_HXX_

#include <mical/samples/kind/sample.hh>
#include <mical/samples/kind/concrete/predefs.hh>

namespace mical {

  template <typename Container_, typename Word_>
  FixedSample<Container_, Word_>::FixedSample()
  {}
  
  template <typename Container_, typename Word_>
  void                      
  FixedSample<Container_, Word_>::add_word(const word_t& word)
  {
    PROXY2(add_word, word, container_);
  }

  template <typename Container_, typename Word_>
  void                         
  FixedSample<Container_, Word_>::del_word(const word_t& word)
  {
    PROXY2(del_word, word, container_);
  }
  
  template <typename Container_, typename Word_>
  size_t                       
  FixedSample<Container_, Word_>::size() const
  {
    return PROXY1(size, container_);
  }

  template <typename Container_, typename Word_>
  bool                         
  FixedSample<Container_, Word_>::empty() const
  {
    return PROXY1(empty, container_);
  }
  
  template <typename Container_, typename Word_>
  typename Container_::iterator               
  FixedSample<Container_, Word_>::begin()
  {
    return PROXY2(begin, container_, kind_iterator::not_const());
  }

  template <typename Container_, typename Word_>
  typename Container_::const_iterator               
  FixedSample<Container_, Word_>::begin() const
  {
    return PROXY2(begin, container_, kind_iterator::not_mutable());
  }

  template <typename Container_, typename Word_>
  typename Container_::reverse_iterator       
  FixedSample<Container_, Word_>::rbegin()
  {
    return PROXY2(rbegin, container_, kind_iterator::not_const());
  }

  
  template <typename Container_, typename Word_>
  typename Container_::iterator               
  FixedSample<Container_, Word_>::end()
  {
    return PROXY2(end, container_, kind_iterator::not_const());
  }

  template <typename Container_, typename Word_>
  typename Container_::const_iterator               
  FixedSample<Container_, Word_>::end() const
  {
    return PROXY2(end, container_, kind_iterator::not_mutable());
  }

  template <typename Container_, typename Word_>
  typename Container_::reverse_iterator       
  FixedSample<Container_, Word_>::rend()
  {
    return PROXY2(rend, container_, kind_iterator::not_const());
  }

  
  template <typename Container_, typename Word_>
  void 
  FixedSample<Container_, Word_>::clear()
  {
    PROXY1(clear, container_);
  }
  
  template <typename Container_, typename Word_>
  Container_
  FixedSample<Container_, Word_>::words() const
  {
    return container_;
  }

  template <typename Container_, typename Word_>
  Container_&
  FixedSample<Container_, Word_>::container()
  {
    return container_;
  }

  template <typename Container_, typename Word_>
  FixedSample<Container_, Word_>&
  FixedSample<Container_, Word_>::
  operator=(const FixedSample<Container_, Word_>& other)
  {
    typedef typename 
      FixedSample<Container_, Word_>::sample_iterator 
      sample_iterator_t;

    this->clear();

    for (sample_iterator_t i = other.begin(); i != other.end(); i++)
      this->add_word(*i);
    
    return this;
  }

  ///////////////////////////////////////////////////////////////////////////

  template <typename Container_, typename Word_>
  bool 
  operator==(const FixedSample<Container_, Word_>& lhs,
	     const FixedSample<Container_, Word_>& rhs)
  {
    return (lhs.words() == rhs.words());
  }
	     

  // in the meaning of inclusion
  template <typename Container_, typename Word_>
  bool
  operator<=(const FixedSample<Container_, Word_>& lhs,
	     const FixedSample<Container_, Word_>& rhs)
  {
    return includes(lhs.begin(), lhs.end(), rhs.begin(), rhs.end());
  }

  // in the meaning of inclusion
  template <typename Container_, typename Word_>
  bool
  operator>=(const FixedSample<Container_, Word_>& lhs,
	     const FixedSample<Container_, Word_>& rhs)
  {
    return includes(lhs.begin(), lhs.end(), rhs.begin(), rhs.end());
  }

} // mical

#endif // MICAL_FIXED_SAMPLE_HXX_
