// is_struct_complete.hxx
//
// Mical, a regular grammar inference library
// Copyright (C) 2003 Maxime Rey
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef   MICAL_IS_STRUCT_COMPLETE_HXX_
#define   MICAL_IS_STRUCT_COMPLETE_HXX_

#include <mical/samples/services/is_struct_complete.hh>

#include <mical/primitives/compute.hh>

#include <vector>
#include <set>
#include <utility>

using namespace vcsn;

using std::set;
using std::vector;
using std::pair;

namespace mical {

  using namespace vcsn;
  
  template <class A, class T, class Container_>
  bool 
  is_sc_sample(const Element<A, T>& a, 
	       const FixedSample<Container_, 
	       typename Element<A, T>::monoid_elt_t>& sample)
  {
    typedef FixedSample<Container_, 
      typename Element<A, T>::monoid_elt_t>  sample_t;
    typedef vector<unsigned>                 possibility_t;
    
    possibility_t possibility(sample.size(), 1);

    bool flag = false;
    typename sample_t::sample_const_iterator it = sample.begin();
    
    is_sc_sample_(a, sample.words(), it, 0, 
		  set<hedge_t>(), set<hstate_t>(),
		  flag);
    
    return flag;
  }

  template <class A, class T, class Kind_, class Container_>
  bool 
  is_sc_sample(const Element<A, T>& a, 
	       const Presentation<Kind_, Container_, 
	       typename Element<A, T>::monoid_elt_t>& sample)
  {
    return is_sc_sample(a, sample.positive());
  }
  
  template <class A, class T, class Container_>
  bool 
  is_sc_sample(const Element<A, T>& a, 
	       Container_ container)
  {
    typedef vector<unsigned>  possibility_t;
    
    possibility_t possibility(container.size(), 1);

    bool flag = false;
    typename Container_::iterator it = container.begin();
    
    is_sc_sample_(a, container, it, 0, 
		  set<hedge_t>(), set<hstate_t>(),
		  flag);
    
    return flag;
  }

  template <class A, class T, class Container_>
  void
  is_sc_sample_(const Element<A, T>& a, 
		const Container_& samples,
		typename Container_::iterator& word,
		unsigned current, 
		set<hedge_t> edges, 
		set<hstate_t> states,
		bool& flag)
  {
    if (flag)
      return;
    
    if (current == samples.size())
      flag = ((edges.size() == a.edges().size()) && (states.size() == a.final().size()));
    else
      {
	Compute_n<A, T, typename Element<A, T>::monoid_elt_t> compute;
	bool not_ok;
	unsigned count = 1;
	
	do 
	  {
 	    set<hedge_t> svg_edges = edges;
	    set<hstate_t> svg_states = states;

	    compute(a, *word, count, edges);
	    
	    int tmp;
	    if ((tmp = compute(a, *word, count, compute_kind::state())) != -1)
		states.insert(tmp);
	    
	    is_sc_sample_(a, samples, ++word, current + 1, edges, states, flag);
	    word--;
	    
	    edges = svg_edges;
	    states = svg_states;

	    not_ok = !flag && (tmp != -1);
	    count++;
	  }
	while (not_ok);
      }
  }


} // mical

#endif // MICAL_IS_STRUCT_COMPLETE_HXX_
