// choose.hxx
//
// Mical, a regular grammar inference library
// Copyright (C) 2003 Maxime Rey
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef MICAL_CHOOSE_HXX_
#define MICAL_CHOOSE_HXX_

#include <mical/tools/choose.hh>

#include <set>

namespace mical {

  using std::set;

  template <typename Container_>
  typename Container_::iterator
  choose(const Container_& c)
  {
    if (c.size() == 0)
      return c.end();

    unsigned end = RANDOM_VAL(c.size());

    typename Container_::iterator res = c.begin();
    for (unsigned i = 0; i < end; i++)
      res++;
    
    return res;
  }

  template <class A, class T>
  hedge_t
  choose_edge(const Element<A, T>& a, hstate_t state)
  {
    set<hedge_t> edges;

    a.deltac(edges, state, delta_kind::edges());

    return *choose(edges);
  }

} // mical

#endif // MICAL_CHOOSE_HXX_
