// predefs_edges.hh
//
// Mical, a regular grammar inference library
// Copyright (C) 2003 Maxime Rey
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef   PREDEFS_EDGES_HH_
# define   PREDEFS_EDGES_HH_

#include <vaucanson/fundamental/element.hh>
#include <vaucanson/automata/concept/automata.hh>

#include <set>

using namespace vcsn;
using std::set;

namespace mical {

  template <class A, class T, class W>
  void add_edge(Element<A, T>& a, hstate_t src, hstate_t dst, W label)
  {
    set<hedge_t> delta;

    a.deltac(delta, src, delta_kind::edges());

    for (typename set<hedge_t>::iterator i = delta.begin(); i != delta.end(); i++)
      if ((a.label_of(*i) == label) && (a.aim_of(*i) == dst))
	return;

    a.add_edge(src, dst, label);
  }

  template <class A, class T, class W>
  void add_letter_edge(Element<A, T>& a, hstate_t src, hstate_t dst, W label)
  {
    set<hedge_t> delta;

    a.deltac(delta, src, delta_kind::edges());

    for (typename set<hedge_t>::iterator i = delta.begin(); i != delta.end(); i++)
      if ((*a.label_of(*i).begin()->first.begin() == label) 
	  && (a.aim_of(*i) == dst))
	return;

    a.add_letter_edge(src, dst, label);
  }

  template <class A, class T>
  void del_edge(Element<A, T>& a, hedge_t h)
  {
    if (a.has_edge(h))
      a.del_edge(h);    
  }


  template <class A, class T>
  hedge_t get_edge(const Element<A, T>& a, hstate_t origin, hstate_t aim)
  {
    set<hedge_t> delta_edge;
    a.rdeltac(delta_edge, aim, delta_kind::edges());
    
    for (typename set<hedge_t>::iterator i = delta_edge.begin(); 
	 i != delta_edge.end(); i++)
      if (a.origin_of(*i) == origin)
	return *i;
    
    return 0;
  }

  template <class A, class T>
  bool 
  equal(const Element<A, T>& a, hedge_t lhs, hedge_t rhs)
  {
    return ((a.origin_of(lhs) == a.origin_of(rhs)) &&
	    (a.aim_of(lhs) == a.aim_of(rhs)) &&
	    (a.label_of(lhs) == a.label_of(rhs)));
  }

} // mical

#endif /* !PREDEFS_EDGES_HH_ */
