// depth_search.hh
//
// Mical, a regular grammar inference library
// Copyright (C) 2003 Maxime Rey
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef MICAL_DEPTH_SEARCH_HH_
# define MICAL_DEPTH_SEARCH_HH_

#include <vaucanson/fundamental/element.hh>
#include <vaucanson/automata/concept/automata.hh>

#include <mical/walkers/info.hh>
#include <mical/walkers/security.hh>

#include <algorithm>

namespace mical {

  template <class A, class T, class Task_, class Secure_>
  void
  depth_search(const Element<A, T>& a, 
	       Task_& task, 
	       security::level<Secure_> k);
  
  template <class A, class T, class Task_, class Secure_>
  void
  depth_search(Element<A, T>& a, 
	       Task_& task, 
	       security::level<Secure_> k);

  template <class A, class T, class Task_, class Secure_>
  void
  depth_search(const Element<A, T>& a, 
	       hstate_t init, 
	       Task_& task, 
	       security::level<Secure_> k);

  template <class A, class T, class Task_, class Secure_>
  void
  depth_search(Element<A, T>& a, 
	       hstate_t init, 
	       Task_& task, 
	       security::level<Secure_> k);

} // mical

#include <mical/walkers/depth_search.hxx>

#endif // MICAL_DEPTH_SEARCH_HH_
