// test_rpni.cc

#include <mical/algorithms/rpni.hh>

#include <vaucanson/algebra/concrete/free_monoid/predefs.hh>
#include <vaucanson/algebra/concrete/semiring/numerical_semiring.hh>
#include <vaucanson/algebra/concrete/series/polynoms.hh>
#include <vaucanson/automata/concept/automata.hh>
#include <vaucanson/tools/usual.hh>

int main(int argc, char** argv)
{
  using namespace std;
  using namespace mical;
  using namespace vcsn;
  using namespace vcsn::algebra;
  using namespace vcsn::algebra::char_letter;

  typedef Automata auto_t;
  typedef 
    utility::ref< 
    AutomatonImpl<labels_are_series,
    Series<NumericalSemiring, Words>,
    polynom<Word::value_t, bool>,
    ManyLinks<polynom<Word::value_t, bool>, NoTag, NoTag>,
    NoTag,
    map> >
    implem_t;

  typedef Element<auto_t, implem_t> automaton_t;

  typedef   FrameWork<
    search_kind::pta, 
    inference_kind::fixed, 
    framework_traits<automaton_t, set<set<hstate_t> >, set<Word>, inference_kind::fixed> > framework_t;

  framework_t fw("toy.d");
  
  automaton_t a = fw.generate();
  a.create();

//   cout << "Initial Automaton" << endl;

//   for (automaton_t::edge_iterator i = a.edges().begin(); i != a.edges().end(); i++)
//     cout << a.origin_of(*i) << " - " << a.aim_of(*i) << " | " << a.label_of(*i).begin()->first << endl;

//   cout << endl;
//   cout << "INITIAL" << endl;
//   for (automaton_t::initial_iterator i = a.initial().begin(); i != a.initial().end(); i++)
//     cout << *i << endl;
//   cout << endl;

//   cout << "FINAL" << endl;
//   for (automaton_t::final_iterator i = a.final().begin(); i != a.final().end(); i++)
//     cout << *i << endl;
//   cout << endl;

  SAVE_AUTOMATON_DOT_SIMPLE("before_rpni", a);
  
  rpni(fw);

  a = fw.generate();

//   cout << "Resulting automaton after application of RPNI algorithm" << endl;

//   for (automaton_t::edge_iterator i = a.edges().begin(); i != a.edges().end(); i++)
//     cout << a.origin_of(*i) << " - " << a.aim_of(*i) << " | " << a.label_of(*i).begin()->first << endl;

//   cout << endl;
//   cout << "INITIAL" << endl;
//   for (automaton_t::initial_iterator i = a.initial().begin(); i != a.initial().end(); i++)
//     cout << *i << endl;
//   cout << endl;

//   cout << "FINAL" << endl;
//   for (automaton_t::final_iterator i = a.final().begin(); i != a.final().end(); i++)
//     cout << *i << endl;
//   cout << endl;

  SAVE_AUTOMATON_DOT_SIMPLE("after_rpni", a);

  return 0;
}
