// test_is_compatible.cc

#include <vaucanson/fundamental/predefs.hh>
#include <vaucanson/algebra/concrete/free_monoid/predefs.hh>
#include <vaucanson/algebra/concrete/semiring/numerical_semiring.hh>
#include <vaucanson/algebra/concrete/series/polynoms.hh>
#include <vaucanson/tools/usual.hh>

#include <mical/internal/is_compatible.hh>
#include <mical/internal/make_mca.hh>

#include <mical/samples/tools/load_sample.hh>

int main(int argc, char** argv)
{
  using namespace std;
  using namespace mical;
  using namespace vcsn;
  using namespace vcsn::algebra;
  using namespace vcsn::algebra::char_letter;

  Alphabet a;

  a.insert('a');
  a.insert('b');
  
  Words free_monoid(a);
  NumericalSemiring ring;
  Series<NumericalSemiring, Words> s(ring, free_monoid);

  typedef Automata auto_t;
  typedef 
    utility::ref< 
    AutomatonImpl<labels_are_series,
    Series<NumericalSemiring, Words>,
    polynom<Word::value_t, bool>,
    ManyLinks<polynom<Word::value_t, bool>, NoTag, NoTag>,
    NoTag,
    map> >
    implem_t;
  
  Element<auto_t, implem_t> A, B;

  A.create();
  B.create();

  hstate_t n_0 = A.add_state();
  hstate_t n_1 = A.add_state();
  hstate_t n_2 = A.add_state();
  hstate_t n_3 = A.add_state();

  A.set_initial(n_0);

  A.add_letter_edge(n_0, n_1, 'b');
  A.add_letter_edge(n_1, n_2, 'a');

  A.set_final(n_2);

  A.add_letter_edge(n_0, n_3, 'b');

  A.set_final(n_3);
  
  SAVE_AUTOMATON_DOT_SIMPLE("automaton1", A);


  ///////////////////////////////////////////////////////////
  FixedSample<set<Word>, Word> sample_neg;

  load_neg_sample(sample_neg, "toy.a");

  cout << is_compatible(A, sample_neg) << endl;
  ///////////////////////////////////////////////////////////


  ///////////////////////////////////////////////////////////
  Presentation<inference_kind::fixed, set<Word>, Word> pres;

  load_presentation(pres, "toy.a");

  make_mca(B, pres);

  SAVE_AUTOMATON_DOT_SIMPLE("automaton2", A);

  cout << is_compatible(B, pres) << endl;
  ///////////////////////////////////////////////////////////
  return 0;
}
