// test_compute_n.cc

#include <vaucanson/fundamental/predefs.hh>
#include <vaucanson/algebra/concrete/free_monoid/predefs.hh>
#include <vaucanson/algebra/concrete/semiring/numerical_semiring.hh>
#include <vaucanson/algebra/concrete/series/polynoms.hh>
#include <vaucanson/tools/usual.hh>

#include <mical/primitives/compute_all.hh>


int main(int argc, char** argv)
{
  using namespace std;
  using namespace mical;
  using namespace vcsn;
  using namespace vcsn::algebra;
  using namespace vcsn::algebra::char_letter;

  Alphabet a;

  a.insert('a');
  a.insert('b');
  
  Words free_monoid(a);
  NumericalSemiring ring;
  Series<NumericalSemiring, Words> s(ring, free_monoid);

  typedef Automata auto_t;
  typedef 
    utility::ref< 
    AutomatonImpl<labels_are_series,
    Series<NumericalSemiring, Words>,
    polynom<Word::value_t, bool>,
    ManyLinks<polynom<Word::value_t, bool>, NoTag, NoTag>,
    NoTag,
    map> >
    implem_t;
  
  Element<auto_t, implem_t> A;

  A.create();

  hstate_t n_0 = A.add_state();
  hstate_t n_1 = A.add_state();
  hstate_t n_2 = A.add_state();
  hstate_t n_3 = A.add_state();
  hstate_t n_4 = A.add_state();

  A.set_initial(n_0);

  A.add_letter_edge(n_0, n_1, 'b');
  A.add_letter_edge(n_1, n_2, 'a');

  A.set_final(n_2);

  A.add_letter_edge(n_0, n_3, 'b');
  A.add_letter_edge(n_3, n_4, 'a');

  A.set_final(n_4);
  
  //  SAVE_AUTOMATON_DOT_SIMPLE("automaton", A);

  cout << compute_all(A, "ba") << endl;
  
  set<hstate_t> res_state;

  compute_all(A, "ba", res_state);

  cout << res_state.size() << " | ";
  copy(res_state.begin(), res_state.end(), ostream_iterator<hstate_t>(cout, " "));
  cout << endl;


  set<hedge_t> res_edge;

  compute_all(A, "ba", res_edge);

  cout << res_edge.size() << " | ";
  copy(res_edge.begin(), res_edge.end(), ostream_iterator<hedge_t>(cout, " "));
  cout << endl;


  return 0;
}
