// test_depth_search.cc

#include <mical/walkers/depth_search.hh>

#include <vaucanson/fundamental/predefs.hh>
#include <vaucanson/algebra/concrete/free_monoid/predefs.hh>
#include <vaucanson/algebra/concrete/semiring/numerical_semiring.hh>
#include <vaucanson/algebra/concrete/series/polynoms.hh>
#include <vaucanson/tools/usual.hh>

#include <vaucanson/automata/concept/automata.hh>
#include <vaucanson/automata/concept/automaton_impl.hh>
#include <vaucanson/automata/concrete/manylinks.hh>

#include <set>
#include <vector>
#include <string>
#include <iostream>
#include <algorithm>

using namespace mical;


//////////////////////////////////////////////////////////////////////
template<class A, class T> 
struct CoAccessible
{
  typedef CoAccessible<A, T>        self_t;
    
  typedef InfoDepth<A, T, self_t>   info_t;

  typedef std::vector<hstate_t>     delta_state_t;

  CoAccessible()
  {
  }

  set<hstate_t>
  operator ()(const Element<A, T>& a, hstate_t start)
  {
    depth_search(a, start, *this, security::safe_t(1));

    return co_acc_states;
  }
    
  void prefix(info_t& info)
  {
    co_acc_states.insert(info.state_);
  }

  void infix(info_t& info)
  {
  }

  void postfix(info_t& info)
  {
  }

  template<typename Container_>
  void 
  delta(info_t& info, Container_& res, hstate_t from)
  {
    info.auto_.rdeltac(res, from, delta_kind::states());
  }

private:

  set<hstate_t> co_acc_states;

};
////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////
template<class A, class T> 
struct Useful
{
  typedef Useful<A, T>   self_t;
    
  typedef Info<A, T, self_t> info_t;

  typedef std::vector<hstate_t>   delta_state_t;

  Useful()
  {
  }

  set<hstate_t>
  operator ()(const Element<A, T>& a)
  {
    depth_search(a, *this, security::safe_t(1));

    set<hstate_t> useful;

    set_intersection(acc_states.begin(), acc_states.end(),
		     co_acc_states.begin(), co_acc_states.end(),
		     inserter(useful, useful.begin()));

    return useful;
  }
    
  void prefix(info_t& info)
  {
    acc_states.insert(info.state_);

    if (info.auto_.is_final(info.state_))
      {
	CoAccessible<A, T> coaccessible;

	co_acc_states = coaccessible(info.auto_, info.state_);
      }
  }

  void infix(info_t& info)
  {
  }

  void postfix(info_t& info)
  {
  }

  template<typename Container_>
  void 
  delta(info_t& info, Container_& res, hstate_t from)
  {
    info.auto_.deltac(res, from, delta_kind::states());
  }

private:

  set<hstate_t> acc_states;
  set<hstate_t> co_acc_states;

};
////////////////////////////////////////////////////////////////////


int main(int argc, char** argv)
{
  using namespace std;
  using namespace mical;
  using namespace vcsn;
  using namespace vcsn::algebra;
  using namespace vcsn::algebra::char_letter;
  using std::set;
  using std::string;
  
  Alphabet a;

  a.insert('a');
  a.insert('b');
  
  Words free_monoid(a);
  NumericalSemiring ring;
  Series<NumericalSemiring, Words> s(ring, free_monoid);

  typedef Automata auto_t;
  typedef 
    utility::ref< 
    AutomatonImpl<labels_are_series,
    Series<NumericalSemiring, Words>,
    polynom<Word::value_t, bool>,
    ManyLinks<polynom<Word::value_t, bool>, NoTag, NoTag>,
    NoTag,
    map> >
    implem_t;
  
  Element<auto_t, implem_t> A;

  A.create();

  hstate_t n_0 = A.add_state();
  hstate_t n_1 = A.add_state();
  hstate_t n_2 = A.add_state();
  hstate_t n_3 = A.add_state();
  hstate_t n_4 = A.add_state();
  hstate_t n_5 = A.add_state();

  A.set_initial(n_0);
  A.set_final(n_3);

  A.add_letter_edge(n_0, n_1, 'b');
  A.add_letter_edge(n_1, n_2, 'a');

  A.add_letter_edge(n_2, n_3, 'b');

  A.add_letter_edge(n_1, n_4, 'a');
  A.add_letter_edge(n_5, n_2, 'a');

  //  SAVE_AUTOMATON_DOT_SIMPLE("automaton", A);

  Useful<auto_t, implem_t> useful;
  
  set<hstate_t> res = useful(A);

  for (set<hstate_t>::iterator i = res.begin(); i != res.end(); i++)
    cout << *i << endl;

  return 0;
}
