// tropical_semiring.hh
//
//
//
// $Id: tropical_semiring.hh 1.6 Wed, 26 Feb 2003 14:16:00 +0100 akim $
// Vaucanson, a generic library for finite state machines.
// Copyright (C) 2001, 2002 Sakarovitch, Lombardy, Poss, Rey and Regis-Gianas.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


#ifndef ALGEBRA_CONCEPT_TROPICAL_SEMIRING_HH
# define ALGEBRA_CONCEPT_TROPICAL_SEMIRING_HH

# include <vaucanson/algebra/concept/semiring_base.hh>

namespace vcsn {

  namespace algebra {

    template <class TropicalKind>
    struct TropicalSemiring;

  } // algebra

  /*! \addtogroup algebra */  /* @{ */
  /*! \addtogroup semiring */ /* @{ */

  /*----------------------------------------------------.
  | MetaSet : traits about the set of tropical semiring |
  `----------------------------------------------------*/
  //! Meta information about the tropical semirings.
  template<class TropicalKind>
  struct MetaSet<TropicalSemiring<TropicalKind> >
  {
    //! Semiring are static structures.
    static const bool dynamic_set = false;
    
    //! We can see a NumericalSemiring as a semigroup with +. It is
    //! an arbitrary decision, it could be mul_kind too.
    typedef add_kind	semigroup_kind;
  };

  /*! @} @} */
  
  namespace algebra {

    /*! \addtogroup algebra */  /* @{ */
    /*! \addtogroup semiring */ /* @{ */

    /*-----------------.
    | Kind of tropical |
    `-----------------*/
    //! Static information about the kind of tropical we are
    //! confronted with.
    struct TropicalMin
    {};

    //! Static information about the kind of tropical we are
    //! confronted with.
    struct TropicalMax
    {};

    /*-----------------.
    | TropicalSemiring |
    `-----------------*/
    //! Structural element that denotes tropical semiring.
    template <class TropicalKind>
    struct TropicalSemiring
      : SemiringBase<TropicalSemiring<TropicalKind> >
    {};

    /*! @} @} */

  } // algebra

  //! \addtogroup algebra 
  //! \@{

  /*---------------------------------------.
  | MetaElement<TropicalSemiring<Kind>, T> |
  `---------------------------------------*/
  //! Services of element of a tropical semiring.
  template<class TropicalKind, typename T>
  struct MetaElement<TropicalSemiring<TropicalKind> , T>
    : MetaElement<SemiringBase<TropicalSemiring<TropicalKind> >, T>
  {};

  //! @}

} // vcsn

#endif // ALGEBRA_CONCEPT_TROPICAL_SEMIRING_HH
